/**
 * trzsz: https://github.com/trzsz/trzsz.js
 * Copyright(c) 2022 Lonny Wong <lonnywong@qq.com>
 * @license MIT
 */

export {};

import { BufferSizeParser } from "./args";
import { checkPathsReadable } from "./nodefs";
import { ArgumentParser, RawTextHelpFormatter } from "argparse";
import { trzszVersion, checkDuplicateNames, TrzszError } from "./comm";

/* eslint-disable require-jsdoc */

function parseArgs() {
  const parser = new ArgumentParser({
    description: "Send file(s), similar to sz and compatible with tmux.",
    formatter_class: RawTextHelpFormatter,
  });
  parser.add_argument("-v", "--version", { action: "version", version: `%(prog)s (trzsz) js ${trzszVersion}` });
  parser.add_argument("-q", "--quiet", { action: "store_true", help: "quiet (hide progress bar)" });
  parser.add_argument("-y", "--overwrite", { action: "store_true", help: "yes, overwrite existing file(s)" });
  parser.add_argument("-b", "--binary", {
    action: "store_true",
    help: "binary transfer mode, faster for binary files",
  });
  parser.add_argument("-e", "--escape", { action: "store_true", help: "escape all known control characters" });
  parser.add_argument("-d", "--directory", { action: "store_true", help: "transfer directories and files" });
  parser.add_argument("-B", "--bufsize", {
    min_size: "1K",
    max_size: "1G",
    default: "10M",
    action: BufferSizeParser,
    metavar: "N",
    help: "max buffer chunk size (1K<=N<=1G). (default: 10M)",
  });
  parser.add_argument("-t", "--timeout", {
    type: "int",
    default: 10,
    metavar: "N",
    help: "timeout ( N seconds ) for each buffer chunk.\nN <= 0 means never timeout. (default: 10)",
  });
  parser.add_argument("file", { nargs: "+", help: "file(s) to be sent" });
  return parser.parse_args();
}

/**
 * tsz main entry
 */
function main() {
  const args = parseArgs();
  console.dir(args);

  try {
    const fileList = checkPathsReadable(args.file, args.directory);
    console.log(fileList);

    if (args.overwrite) {
      checkDuplicateNames(fileList);
    }
  } catch (err) {
    console.log(TrzszError.getErrorMessage(err));
    return;
  }
}

main();

// redirect to the python version
console.log(
  "\nThe Node.js version is under development.\n" +
    "Please use the Python version instead.\n" +
    "GitHub: https://github.com/trzsz/trzsz\n"
);

const bin = "/usr/local/bin/tsz";
const fs = require("fs");
if (!fs.existsSync(bin)) {
  process.exit();
}
const script = fs.readFileSync(bin, "utf-8");
if (script.startsWith("#!/usr/bin/env node")) {
  process.exit();
}

const { spawnSync } = require("child_process");
spawnSync(bin, process.argv.slice(2), { stdio: "inherit" });
